
open Mucalc;;

(* Complete automaton tester *)
module MuCalc_ResultType = 
    struct
        type t = mu_formula
        let equal x y = (x = y)
        let to_string x = mu_formula_to_string x
    end;;

module MuCalcTest = Test.Tester(MuCalc_ResultType);;


(* Tests defined below *)
let test_mucalc () = 
    print_string "Testing Mucalc:\n\n";
    MuCalcTest.run_tests ();
    print_string "Done.\n\n";;


(*****************************************************************************************)

let test1 () = 
    let f = Bin(Prop("x"),And,Bin(Var("Z"),Or,Prop("x"))) in
    normalise_formula f;;

let expected1 = Bin(Prop("x"),And,Bin(Var("Z"),Or,Prop("x")));;

MuCalcTest.add_test "Test 1: Normalise, no change." test1 expected1;;


let test2 () = 
    let f = Bin(Mu("Z",Bin(Var("Z"),And,Var("Y"))),
                And,
                Nu("Y",Bin(Var("Z"),And,Var("Y")))) in
    normalise_formula f;;

let expected2 = Bin(Mu("Z_0",Bin(Var("Z_0"),And,Var("Y"))),
                    And,
                    Nu("Y_1",Bin(Var("Z"),And,Var("Y_1"))));;

MuCalcTest.add_test "Test 2: Normalise, two bound vars, not nested." test2 expected2;;



let test3 () = 
    let f = Mu("Z",Bin(Var("Z"),And,Nu("Z",Var("Z")))) in
    normalise_formula f;;

let expected3 =  Mu("Z_0",Bin(Var("Z_0"),And,Nu("Z_1",Var("Z_1"))));;

MuCalcTest.add_test "Test 3: Normalise, two bound vars, nested." test3 expected3;;




let test4 () = 
    let f = Un(Not,Bin(Prop("x"),And,Bin(Prop("z"),Or,Prop("x")))) in
    normalise_formula f;;

let expected4 = Bin(NegProp("x"),Or,Bin(NegProp("z"),And,NegProp("x")));;

MuCalcTest.add_test "Test 4: Normalise, negation, no vars." test4 expected4;;




let test5 () = 
    let f = Un(Not,Bin(Prop("x"),And,Un(Not,Bin(Prop("z"),Or,Prop("x"))))) in
    normalise_formula f;;

let expected5 = Bin(NegProp("x"),Or,Bin(Prop("z"),Or,Prop("x")));;

MuCalcTest.add_test "Test 5: Normalise, double negation, no vars." test5 expected5;;



let test6 () = 
    let f = Un(Not,Mu("Z",Bin(Var("Z"),And,Nu("Z",Var("Z"))))) in
    normalise_formula f;;

let expected6 =  Nu("Z_0",Bin(Var("Z_0"),Or,Mu("Z_1",Var("Z_1"))));;

MuCalcTest.add_test "Test 6: Normalise, negate nested bound vars." test6 expected6;;



let test7 () = 
    let f = Bin(Prop("x"),Implies,Prop("y")) in
    normalise_formula f;;

let expected7 = Bin(NegProp("x"),Or,Prop("y"));;

MuCalcTest.add_test "Test 7: Normalise, implication." test7 expected7;;


let test8 () = 
    let f = Un(Not,Bin(Prop("x"),Implies,Prop("y"))) in
    normalise_formula f;;

let expected8 = Bin(Prop("x"),And,NegProp("y"));;

MuCalcTest.add_test "Test 8: Normalise, negated implication." test8 expected8;;
